import 'package:flutter/material.dart';
import 'package:getx_extended/export.dart';

class RxPageable<T> extends RxList<T> {
  static var pageSize = 20;
  static var defaultFirstPage = 1;

  Future<List<T>> Function(int page, int size) dataSource;

  RxPageable({
    required this.dataSource,
  }) : super() {
    refetch();
  }

  late int _currentPage;
  late bool _isEnded;
  late bool _isLoading;

  Future refetch() async {
    _currentPage = defaultFirstPage;
    _isEnded = false;
    _isLoading = false;
    await next();
  }

  T item(int index) {
    if (index + 6 > value.length) {
      next();
    }
    return value[index];
  }

  Future next() async {
    WidgetsBinding.instance?.addPostFrameCallback((timeStamp) async {
      if (_isEnded || _isLoading) return;
      try {
        _isLoading = true;
        final list = await dataSource(_currentPage, pageSize);
        if (_currentPage == defaultFirstPage) {
          assignAll(list);
        } else {
          addAllIf(true, list.toList());
        }
        _currentPage += 1;
        _isEnded = pageSize > list.length;
      } finally {
        _isLoading = false;
      }
    });
  }
}
