import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_keyboard_visibility/flutter_keyboard_visibility.dart';
import 'package:getx_extended/controller/source/task_controller.dart';
import 'package:getx_extended/export.dart';

class QuickRoot extends StatefulWidget {
  Widget child;
  Widget Function(bool isKeyboardVisible) keyboardToolbar;
  Widget loading;
  Future Function(dynamic error) consumeError;

  QuickRoot({
    required this.child,
    required this.keyboardToolbar,
    required this.loading,
    required this.consumeError,
  });

  @override
  _QuickRootState createState() => _QuickRootState();
}

class _QuickRootState extends State<QuickRoot> {
  late TaskController _task = Get.find();

  late StreamSubscription _errorStream;

  @override
  void initState() {
    super.initState();
    _errorStream = _task.errors.listen((_) => _checkError());
  }

  @override
  void dispose() {
    _errorStream.cancel();
    super.dispose();
  }

  dynamic _currentError;

  void _checkError() {
    final error = _task.errors.firstOrNull;
    if (error == null || _currentError == error) return;
    _currentError = error;
    WidgetsBinding.instance?.addPostFrameCallback((timeStamp) async {
      await widget.consumeError(error);
      _currentError = null;
      _task.dismiss(error);
    });
  }

  @override
  Widget build(BuildContext context) => Scaffold(
        body: SafeArea(
          child: KeyboardVisibilityBuilder(
            builder: (_, isKeyboardVisible) => Column(
              children: [
                Expanded(
                  child: Stack(
                    children: [
                      widget.child,
                      ObxValue<RxBool>(
                        (rx) => rx.value ? widget.loading : Container(),
                        _task.isProcessing,
                      ),
                    ],
                  ),
                ),
                widget.keyboardToolbar(isKeyboardVisible)
              ],
            ),
          ),
        ),
      );
}
